import 'package:firebase_messaging/firebase_messaging.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:flutter_platform_widgets/flutter_platform_widgets.dart';
import 'package:overlay_support/overlay_support.dart';
import 'package:provider/provider.dart';
import 'package:sunny_core_widgets/intl/sunny_intl.dart';
import 'package:sunny_essentials/provided.dart';
import 'package:sunny_essentials/state.dart';
import 'package:sunny_essentials/theme.dart';
import 'package:sunny_dart/sunny_get.dart';
// import 'lifecycle_extensions.dart';
import 'package:sunny_lifecycle/app_info.dart';
import 'package:sunny_lifecycle/auth_state.dart';
import 'package:sunny_lifecycle/push_notification_service.dart';
import 'package:sunny_lifecycle/sunny_injector.dart';
import 'package:sunny_sdk_core/sunny_sdk_core.dart';
import 'package:sunny_core_widgets/routes.dart';

import 'f_routing.dart';
import 'lifecycle_main.dart';

class SunnyApp extends StatefulWidget {
  final AppInfo appInfo;
  final AppBootstrap app;

  const SunnyApp(this.appInfo, {required this.app, Key? key}) : super(key: key);

  @override
  _SunnyAppState createState() => _SunnyAppState();
}

class _SunnyAppState extends BaseState<SunnyApp> {
  FRouteState? routeState;
  FRouterDelegate? _routerDelegate;
  FRouterRootProvider? _routeProvider;
  PushNotificationService? _push;

  @override
  void initState() {
    super.initState();

    _push = PushNotificationService(
      context,
      widget.appInfo,
      userStateStream: globalUserStateChanged,
      convertMessage: app.pushMessageBodyConverter,
      sendDevicePushToken: app.sendDevicePushToken,
    );

    SunnyInjector.of(context).register(_push, isSilent: true);
    app.initWithinApp.call(context);

    routeState = FRouteState(
      loggedIn: sunny.get<IAuthState>(context: context).current,
    );

    var initialRoute = RouteInformation(
      location: "/splash",
      state: routeState,
    );
    // _platformProvider = PlatformRouteInformationProvider(
    //   initialRouteInformation: initialRoute,
    // );

    _routeProvider = FRouterRootProvider(
      initialRoute: initialRoute,
    );

    _routerDelegate = FRouterDelegate(
      routeState,
      app.router,
      tabs: app.tabs,
      userChanges: app.lifecycleInit!.authState.userStateStream,
      loggedInWidget: app.loggedInWidget,
      splashPageWidget: app.splashPageWidget,
      loggedOutWidget: app.loggedOutWidget,
      navigatorKey: rootNavigatorKey,
    );

    _routerDelegate!.addListener(() {
      log.info("Rebuilding Sunny App because delegate changed");
    });
  }

  @override
  void dispose() {
    _routerDelegate!.dispose();
    routeState!.dispose();
    _routeProvider!.dispose();
    // _platformProvider.dispose();
    super.dispose();
  }

  AppBootstrap get app => widget.app;

  @override
  Widget build(BuildContext context) {
    /// Set up all the scaling business

    final brightness = context.get<Brightness>();
    final layoutInfo = context.layoutInfo;
    app.screenInit.call(context, layoutInfo);

    final themes = context.themes;
    return MultiProvider(
      providers: [
        Provider.value(value: _push),
        ChangeNotifierProvider.value(value: routeState),
        ChangeNotifierProvider.value(value: _routerDelegate),
        ChangeNotifierProvider.value(value: _routeProvider),
      ],
      child: OverlaySupport(
        key: Key("outerOverlaySupport"),
        child: PlatformProvider(
          settings: PlatformSettingsData(
            platformStyle: PlatformStyleData(
              web: layoutInfo.platformStyle,
            ),
          ),
          builder: (context) {
            return app.serviceResolver.register(
              context,
              [
                // Inst.constant(
                //     const SunnyFormTheme(starForRequiredFields: false)),
                if (!kIsWeb) Inst.instance(FirebaseMessaging.instance),
                Inst.factory((_) => SunnyIntl.load(enUs)),
              ],
              key: Key("root-context-$brightness"),
              child: app.buildPlatformApp(
                context,
                PlatformApp.router(
                  debugShowCheckedModeBanner: false,
                  title: app.appTitle,
                  routeInformationParser: FRouterInformationParser(
                    app.router,
                    app,
                  ),

                  /// The router doesn't listen to the delegate
                  routerDelegate: _routerDelegate,
                  // backButtonDispatcher: RootBackButtonDispatcher(),
                  // routeInformationProvider: _routeProvider,
                  // onGenerateRoute: app.router?.onGenerateRoute,
                  // initialRoute: "/",
                  localizationsDelegates: [
                    DefaultMaterialLocalizations.delegate,
                  ],
                  cupertino: (context, platform) => CupertinoAppRouterData(
                    theme: context.sub(),
                    // color: TrippiColors.primaryColor,
                    color: sunnyColors.primaryColor,
                    title: app.appTitle,
                  ),
                  material: (context, platform) => MaterialAppRouterData(
                    theme: themes.lightTheme,
                    color: themes.lightTheme.primaryColor,
                    darkTheme: app.isChangeBrightness
                        ? themes.darkTheme
                        : themes.lightTheme,
                  ),
                ),
              ),
            );
          },
        ),
      ),
    );
  }
}
